/*
 * main_VL53L0X.c
 *
 * Created: 08.03.2018 09:30:06
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "VL53L0X.h"

// Uncomment this line to use long range mode. This
// increases the sensitivity of the sensor and extends its
// potential range, but increases the likelihood of getting
// an inaccurate reading because of reflections from objects
// other than the intended target. It works best in dark
// conditions.

#define LONG_RANGE


// Uncomment ONE of these two lines to get
// - higher speed at the cost of lower accuracy OR
// - higher accuracy at the cost of lower speed

//#define HIGH_SPEED
#define HIGH_ACCURACY

int main(void)
{
	cli();
	
	// initialize I2C library
	i2c_init();	
	
	sei();
	
	init(true);
	setTimeout(500);
	
#if defined LONG_RANGE
	// lower the return signal rate limit (default is 0.25 MCPS)
	setSignalRateLimit(0.1);
	// increase laser pulse periods (defaults are 14 and 10 PCLKs)
	setVcselPulsePeriod(VcselPeriodPreRange, 18);
	setVcselPulsePeriod(VcselPeriodFinalRange, 14);
#endif
	
#if defined HIGH_SPEED
	// reduce timing budget to 20 ms (default is about 33 ms)
	setMeasurementTimingBudget(20000);
#elif defined HIGH_ACCURACY
	// increase timing budget to 200 ms
	setMeasurementTimingBudget(200000);
#endif	
	volatile uint16_t range;
	
    while(1)
    {
		range = readRangeSingleMillimeters();

		if (timeoutOccurred())
		{
			range = 0xFFFF;
		}
    }
}